@@ -94,9 +94,18 @@ async fn fetch_multiple<R: Runtime>(
 
     let mut useful_responses = Vec::new();
     for r in responses {
+        // TODO: on some error cases we might want to stop using this source.
         match r {
-            Ok(x) => useful_responses.push(x),
-            // TODO: in this case we might want to stop using this source.
+            Ok((request, response)) => {
+                if response.status_code() == 200 {
+                    useful_responses.push((request, response));
+                } else {
+                    trace!(
+                        "cache declined request; reported status {:?}",
+                        response.status_code()
+                    );
+                }
+            }
             Err(e) => warn!("error while downloading: {:?}", e),
         }
     }
@@ -122,6 +131,11 @@ async fn load_once<R: Runtime>(
         let documents = load_all(dirmgr, missing)?;
         state.add_from_cache(documents, dirmgr.store_if_rw())
     };
+
+    if matches!(outcome, Ok(true)) {
+        dirmgr.update_status(state.bootstrap_status());
+    }
+
     outcome
 }
 
@@ -172,7 +186,8 @@ async fn download_attempt<R: Runtime>(
     let missing = state.missing_docs();
     let fetched = fetch_multiple(Arc::clone(dirmgr), missing, parallelism).await?;
     for (client_req, dir_response) in fetched {
-        let text = String::from_utf8(dir_response.into_output())?;
+        let text =
+            String::from_utf8(dir_response.into_output()).map_err(Error::BadUtf8FromDirectory)?;
         match dirmgr.expand_response_text(&client_req, text) {
             Ok(text) => {
                 let outcome = state.add_from_download(&text, &client_req, Some(&dirmgr.store));
@@ -189,6 +204,10 @@ async fn download_attempt<R: Runtime>(
         }
     }
 
+    if changed {
+        dirmgr.update_status(state.bootstrap_status());
+    }
+
     Ok(changed)
 }
 
@@ -325,8 +344,9 @@ fn no_more_than_a_week_from(now: SystemTime, v: Option<SystemTime>) -> SystemTim
 mod test {
     #![allow(clippy::unwrap_used)]
     use super::*;
+    use crate::storage::DynStore;
     use crate::test::new_mgr;
-    use crate::{DownloadSchedule, SqliteStore};
+    use crate::DownloadSchedule;
     use std::convert::TryInto;
     use std::sync::Mutex;
     use tor_netdoc::doc::microdesc::MdDigest;
@@ -391,6 +411,9 @@ mod test {
         fn describe(&self) -> String {
             format!("{:?}", &self)
         }
+        fn bootstrap_status(&self) -> crate::event::DirStatus {
+            crate::event::DirStatus::default()
+        }
         fn is_ready(&self, ready: Readiness) -> bool {
             match (ready, self.second_time_around) {
                 (_, false) => false,
@@ -420,7 +443,7 @@ mod test {
         fn add_from_cache(
             &mut self,
             docs: HashMap<DocId, DocumentText>,
-            _storage: Option<&Mutex<SqliteStore>>,
+            _storage: Option<&Mutex<DynStore>>,
         ) -> Result<bool> {
             let mut changed = false;
             for id in docs.keys() {
@@ -437,7 +460,7 @@ mod test {
             &mut self,
             text: &str,
             _request: &ClientRequest,
-            _storage: Option<&Mutex<SqliteStore>>,
+            _storage: Option<&Mutex<DynStore>>,
         ) -> Result<bool> {
             let mut changed = false;
             for token in text.split_ascii_whitespace() {
@@ -480,7 +503,7 @@ mod test {
                 let mut store = mgr.store_if_rw().unwrap().lock().unwrap();
                 for h in [H1, H2, H3, H4, H5] {
                     store
-                        .store_microdescs(vec![("ignore", &h)], SystemTime::now())
+                        .store_microdescs(&[("ignore", &h)], SystemTime::now())
                         .unwrap();
                 }
             }
@@ -513,7 +536,7 @@ mod test {
                 let mut store = mgr.store_if_rw().unwrap().lock().unwrap();
                 for h in [H1, H2, H3] {
                     store
-                        .store_microdescs(vec![("ignore", &h)], SystemTime::now())
+                        .store_microdescs(&[("ignore", &h)], SystemTime::now())
                         .unwrap();
                 }
             }
