@@ -16,19 +16,22 @@ use std::fmt::Debug;
 use std::sync::{Arc, Mutex, Weak};
 use std::time::{Duration, SystemTime};
 use time::OffsetDateTime;
+use tor_error::internal;
 use tor_netdir::{MdReceiver, NetDir, PartialNetDir};
 use tor_netdoc::doc::netstatus::Lifetime;
 use tracing::{info, warn};
 
-use crate::DirEvent;
+use crate::event::{DirStatus, DirStatusInner};
+
+use crate::storage::{DynStore, EXPIRATION_DEFAULTS};
 use crate::{
     docmeta::{AuthCertMeta, ConsensusMeta},
     retry::DownloadSchedule,
     shared_ref::SharedMutArc,
-    storage::sqlite::SqliteStore,
     CacheUsage, ClientRequest, DirMgrConfig, DirState, DocId, DocumentText, Error, Readiness,
     Result,
 };
+use crate::{DirEvent, DocSource};
 use tor_checkable::{ExternallySigned, SelfSigned, Timebound};
 use tor_llcrypto::pk::rsa::RsaIdentity;
 use tor_netdoc::doc::{
@@ -113,6 +116,14 @@ pub(crate) struct GetConsensusState<DM: WriteNetDir> {
     /// How should we get the consensus from the cache, if at all?
     cache_usage: CacheUsage,
 
+    /// If present, a time after which we want our consensus to have
+    /// been published.
+    //
+    // TODO: This is not yet used everywhere it could be.  In the future maybe
+    // it should be inserted into the DocId::LatestConsensus  alternative rather
+    // than being recalculated in make_consensus_request,
+    after: Option<SystemTime>,
+
     /// If present, our next state.
     ///
     /// (This is present once we have a consensus.)
@@ -133,18 +144,25 @@ impl<DM: WriteNetDir> GetConsensusState<DM> {
     /// Create a new GetConsensusState from a weak reference to a
     /// directory manager and a `cache_usage` flag.
     pub(crate) fn new(writedir: Weak<DM>, cache_usage: CacheUsage) -> Result<Self> {
-        let authority_ids: Vec<_> = if let Some(writedir) = Weak::upgrade(&writedir) {
-            writedir
+        let (authority_ids, after) = if let Some(writedir) = Weak::upgrade(&writedir) {
+            let ids: Vec<_> = writedir
                 .config()
                 .authorities()
                 .iter()
                 .map(|auth| *auth.v3ident())
-                .collect()
+                .collect();
+            let after = writedir
+                .netdir()
+                .get()
+                .map(|nd| nd.lifetime().valid_after());
+
+            (ids, after)
         } else {
             return Err(Error::ManagerDropped);
         };
         Ok(GetConsensusState {
             cache_usage,
+            after,
             next: None,
             authority_ids,
             writedir,
@@ -181,6 +199,13 @@ impl<DM: WriteNetDir> DirState for GetConsensusState<DM> {
     fn can_advance(&self) -> bool {
         self.next.is_some()
     }
+    fn bootstrap_status(&self) -> DirStatus {
+        if let Some(next) = &self.next {
+            next.bootstrap_status()
+        } else {
+            DirStatusInner::NoConsensus { after: self.after }.into()
+        }
+    }
     fn dl_config(&self) -> Result<DownloadSchedule> {
         if let Some(wd) = Weak::upgrade(&self.writedir) {
             Ok(*wd.config().schedule().retry_consensus())
@@ -191,7 +216,7 @@ impl<DM: WriteNetDir> DirState for GetConsensusState<DM> {
     fn add_from_cache(
         &mut self,
         docs: HashMap<DocId, DocumentText>,
-        _storage: Option<&Mutex<SqliteStore>>,
+        _storage: Option<&Mutex<DynStore>>,
     ) -> Result<bool> {
         let text = match docs.into_iter().next() {
             None => return Ok(false),
@@ -205,16 +230,18 @@ impl<DM: WriteNetDir> DirState for GetConsensusState<DM> {
             _ => return Err(Error::Unwanted("Not an md consensus")),
         };
 
-        self.add_consensus_text(true, text.as_str()?)
+        let source = DocSource::LocalCache;
+        self.add_consensus_text(source, text.as_str().map_err(Error::BadUtf8InCache)?)
             .map(|meta| meta.is_some())
     }
     fn add_from_download(
         &mut self,
         text: &str,
         _request: &ClientRequest,
-        storage: Option<&Mutex<SqliteStore>>,
+        storage: Option<&Mutex<DynStore>>,
     ) -> Result<bool> {
-        if let Some(meta) = self.add_consensus_text(false, text)? {
+        let source = DocSource::DirServer {};
+        if let Some(meta) = self.add_consensus_text(source, text)? {
             if let Some(store) = storage {
                 let mut w = store.lock().expect("Directory storage lock poisoned");
                 w.store_consensus(meta, ConsensusFlavor::Microdesc, true, text)?;
@@ -244,12 +271,13 @@ impl<DM: WriteNetDir> GetConsensusState<DM> {
     /// correct, or if it is ill-formed.
     fn add_consensus_text(
         &mut self,
-        from_cache: bool,
+        source: DocSource,
         text: &str,
     ) -> Result<Option<&ConsensusMeta>> {
         // Try to parse it and get its metadata.
         let (consensus_meta, unvalidated) = {
-            let (signedval, remainder, parsed) = MdConsensus::parse(text)?;
+            let (signedval, remainder, parsed) =
+                MdConsensus::parse(text).map_err(|e| Error::from_netdoc(source.clone(), e))?;
             let now = current_time(&self.writedir)?;
             if let Ok(timely) = parsed.check_valid_at(&now) {
                 let meta = ConsensusMeta::from_unvalidated(signedval, remainder, &timely);
@@ -279,7 +307,7 @@ impl<DM: WriteNetDir> GetConsensusState<DM> {
 
         self.next = Some(GetCertsState {
             cache_usage: self.cache_usage,
-            from_cache,
+            consensus_source: source,
             unvalidated,
             consensus_meta,
             missing_certs: desired_certs,
@@ -310,9 +338,8 @@ impl<DM: WriteNetDir> GetConsensusState<DM> {
 struct GetCertsState<DM: WriteNetDir> {
     /// The cache usage we had in mind when we began.  Used to reset.
     cache_usage: CacheUsage,
-    /// True iff we loaded the consensus from our cache.
-    #[allow(dead_code)]
-    from_cache: bool,
+    /// Where did we get our consensus?
+    consensus_source: DocSource,
     /// The consensus that we are trying to validate.
     unvalidated: UnvalidatedMdConsensus,
     /// Metadata for the consensus.
@@ -347,6 +374,16 @@ impl<DM: WriteNetDir> DirState for GetCertsState<DM> {
     fn can_advance(&self) -> bool {
         self.unvalidated.key_is_correct(&self.certs[..]).is_ok()
     }
+    fn bootstrap_status(&self) -> DirStatus {
+        let n_certs = self.certs.len();
+        let n_missing_certs = self.missing_certs.len();
+        let total_certs = n_missing_certs + n_certs;
+        DirStatusInner::FetchingCerts {
+            lifetime: self.consensus_meta.lifetime().clone(),
+            n_certs: (n_certs as u16, total_certs as u16),
+        }
+        .into()
+    }
     fn dl_config(&self) -> Result<DownloadSchedule> {
         if let Some(wd) = Weak::upgrade(&self.writedir) {
             Ok(*wd.config().schedule().retry_certs())
@@ -357,14 +394,17 @@ impl<DM: WriteNetDir> DirState for GetCertsState<DM> {
     fn add_from_cache(
         &mut self,
         docs: HashMap<DocId, DocumentText>,
-        _storage: Option<&Mutex<SqliteStore>>,
+        _storage: Option<&Mutex<DynStore>>,
     ) -> Result<bool> {
         let mut changed = false;
         // Here we iterate over the documents we want, taking them from
         // our input and remembering them.
         for id in &self.missing_docs() {
             if let Some(cert) = docs.get(id) {
-                let parsed = AuthCert::parse(cert.as_str()?)?.check_signature()?;
+                let text = cert.as_str().map_err(Error::BadUtf8InCache)?;
+                let parsed = AuthCert::parse(text)
+                    .map_err(|e| Error::from_netdoc(DocSource::LocalCache, e))?
+                    .check_signature()?;
                 let now = current_time(&self.writedir)?;
                 if let Ok(cert) = parsed.check_valid_at(&now) {
                     self.missing_certs.remove(cert.key_ids());
@@ -381,11 +421,11 @@ impl<DM: WriteNetDir> DirState for GetCertsState<DM> {
         &mut self,
         text: &str,
         request: &ClientRequest,
-        storage: Option<&Mutex<SqliteStore>>,
+        storage: Option<&Mutex<DynStore>>,
     ) -> Result<bool> {
         let asked_for: HashSet<_> = match request {
             ClientRequest::AuthCert(a) => a.keys().collect(),
-            _ => return Err(Error::BadArgument("Mismatched request")),
+            _ => return Err(internal!("expected an AuthCert request").into()),
         };
 
         let mut newcerts = Vec::new();
@@ -447,7 +487,11 @@ impl<DM: WriteNetDir> DirState for GetCertsState<DM> {
     }
     fn advance(self: Box<Self>) -> Result<Box<dyn DirState>> {
         if self.can_advance() {
-            let validated = self.unvalidated.check_signature(&self.certs[..])?;
+            let consensus_source = self.consensus_source.clone();
+            let validated = self
+                .unvalidated
+                .check_signature(&self.certs[..])
+                .map_err(|e| Error::from_netdoc(consensus_source, e))?;
             Ok(Box::new(GetMicrodescsState::new(
                 self.cache_usage,
                 validated,
@@ -476,6 +520,8 @@ struct GetMicrodescsState<DM: WriteNetDir> {
     cache_usage: CacheUsage,
     /// The digests of the microdescriptors we are missing.
     missing: HashSet<MdDigest>,
+    /// Total number of microdescriptors listed in the consensus.
+    n_microdescs: usize,
     /// The dirmgr to inform about a usable directory.
     writedir: Weak<DM>,
     /// The current status of our netdir, if it is not yet ready to become the
@@ -490,6 +536,11 @@ struct GetMicrodescsState<DM: WriteNetDir> {
     /// find a new one.  Since this is randomized, we only compute it
     /// once.
     reset_time: SystemTime,
+    /// If true, we should tell the storage to expire any outdated
+    /// information when we finish getting a usable consensus.
+    ///
+    /// Only cleared for testing.
+    expire_when_complete: bool,
 }
 
 /// A network directory that is not yet ready to become _the_ current network directory.
@@ -552,6 +603,7 @@ impl<DM: WriteNetDir> GetMicrodescsState<DM> {
         writedir: Weak<DM>,
     ) -> Result<Self> {
         let reset_time = consensus.lifetime().valid_until();
+        let n_microdescs = consensus.relays().len();
 
         let partial_dir = match Weak::upgrade(&writedir) {
             Some(wd) => {
@@ -569,12 +621,14 @@ impl<DM: WriteNetDir> GetMicrodescsState<DM> {
         let missing = partial_dir.missing_microdescs().map(Clone::clone).collect();
         let mut result = GetMicrodescsState {
             cache_usage,
+            n_microdescs,
             missing,
             writedir,
             partial: Some(PendingNetDir::Partial(partial_dir)),
             meta,
             newly_listed: Vec::new(),
             reset_time,
+            expire_when_complete: true,
         };
 
         result.consider_upgrade();
@@ -634,14 +688,16 @@ impl<DM: WriteNetDir> GetMicrodescsState<DM> {
     /// storage.
     ///
     /// Called when a consensus is no longer pending.
-    fn mark_consensus_usable(&self, storage: Option<&Mutex<SqliteStore>>) -> Result<()> {
+    fn mark_consensus_usable(&self, storage: Option<&Mutex<DynStore>>) -> Result<()> {
         if let Some(store) = storage {
             let mut store = store.lock().expect("Directory storage lock poisoned");
             info!("Marked consensus usable.");
             store.mark_consensus_usable(&self.meta)?;
             // Now that a consensus is usable, older consensuses may
             // need to expire.
-            store.expire_all()?;
+            if self.expire_when_complete {
+                store.expire_all(&EXPIRATION_DEFAULTS)?;
+            }
         }
         Ok(())
     }
@@ -666,6 +722,15 @@ impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
     fn can_advance(&self) -> bool {
         false
     }
+    fn bootstrap_status(&self) -> DirStatus {
+        let n_present = self.n_microdescs - self.missing.len();
+        DirStatusInner::Validated {
+            lifetime: self.meta.lifetime().clone(),
+            n_mds: (n_present as u32, self.n_microdescs as u32),
+            usable: self.is_ready(Readiness::Usable),
+        }
+        .into()
+    }
     fn dl_config(&self) -> Result<DownloadSchedule> {
         if let Some(wd) = Weak::upgrade(&self.writedir) {
             Ok(*wd.config().schedule().retry_microdescs())
@@ -676,7 +741,7 @@ impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
     fn add_from_cache(
         &mut self,
         docs: HashMap<DocId, DocumentText>,
-        storage: Option<&Mutex<SqliteStore>>,
+        storage: Option<&Mutex<DynStore>>,
     ) -> Result<bool> {
         let mut microdescs = Vec::new();
         for (id, text) in docs {
@@ -685,7 +750,7 @@ impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
                     warn!("Bug: loaded a microdesc that we didn't want from the cache.");
                     continue;
                 }
-                if let Ok(md) = Microdesc::parse(text.as_str()?) {
+                if let Ok(md) = Microdesc::parse(text.as_str().map_err(Error::BadUtf8InCache)?) {
                     if md.digest() == &digest {
                         microdescs.push(md);
                         continue;
@@ -708,12 +773,12 @@ impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
         &mut self,
         text: &str,
         request: &ClientRequest,
-        storage: Option<&Mutex<SqliteStore>>,
+        storage: Option<&Mutex<DynStore>>,
     ) -> Result<bool> {
         let requested: HashSet<_> = if let ClientRequest::Microdescs(req) = request {
             req.digests().collect()
         } else {
-            return Err(Error::BadArgument("Mismatched request"));
+            return Err(internal!("expected a microdesc request").into());
         };
         let mut new_mds = Vec::new();
         for anno in MicrodescReader::new(text, &AllowAnnotations::AnnotationsNotAllowed).flatten() {
@@ -734,14 +799,20 @@ impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
 
         let mark_listed = self.meta.lifetime().valid_after();
         if let Some(store) = storage {
-            let mut s = store.lock().expect("Directory storage lock poisoned");
+            let mut s = store
+                .lock()
+                //.get_mut()
+                .expect("Directory storage lock poisoned");
             if !self.newly_listed.is_empty() {
-                s.update_microdescs_listed(self.newly_listed.iter(), mark_listed)?;
+                s.update_microdescs_listed(&self.newly_listed, mark_listed)?;
                 self.newly_listed.clear();
             }
             if !new_mds.is_empty() {
                 s.store_microdescs(
-                    new_mds.iter().map(|(txt, md)| (&txt[..], md.digest())),
+                    &new_mds
+                        .iter()
+                        .map(|(text, md)| (*text, md.digest()))
+                        .collect::<Vec<_>>(),
                     mark_listed,
                 )?;
             }
@@ -864,12 +935,13 @@ mod test {
         }
     }
 
-    /// Makes a memory-backed SqliteStore.
-    fn temp_store() -> (TempDir, Mutex<SqliteStore>) {
+    /// Makes a memory-backed storage.
+    fn temp_store() -> (TempDir, Mutex<DynStore>) {
         let tempdir = TempDir::new().unwrap();
-        let conn = rusqlite::Connection::open_in_memory().unwrap();
-        let store = SqliteStore::from_conn(conn, tempdir.path()).unwrap();
-        (tempdir, Mutex::new(store))
+
+        let store = crate::storage::SqliteStore::from_path(tempdir.path(), false).unwrap();
+
+        (tempdir, Mutex::new(Box::new(store)))
     }
 
     struct DirRcv {
@@ -1005,6 +1077,9 @@ mod test {
         // Basic properties: it doesn't want to reset.
         assert!(state.reset_time().is_none());
 
+        // Its starting DirStatus is "fetching a consensus".
+        assert_eq!(state.bootstrap_status().to_string(), "fetching a consensus");
+
         // Download configuration is simple: only 1 request can be done in
         // parallel.  It uses a consensus retry schedule.
         let retry = state.dl_config().unwrap();
@@ -1027,7 +1102,7 @@ mod test {
         let req = tor_dirclient::request::ConsensusRequest::new(ConsensusFlavor::Microdesc);
         let req = crate::docid::ClientRequest::Consensus(req);
         let outcome = state.add_from_download("this isn't a consensus", &req, Some(&store));
-        assert!(matches!(outcome, Err(Error::NetDocError(_))));
+        assert!(matches!(outcome, Err(Error::NetDocError { .. })));
         // make sure it wasn't stored...
         assert!(store
             .lock()
@@ -1111,6 +1186,12 @@ mod test {
         let retry = state.dl_config().unwrap();
         assert_eq!(&retry, DownloadScheduleConfig::default().retry_certs());
 
+        // Bootstrap status okay?
+        assert_eq!(
+            state.bootstrap_status().to_string(),
+            "fetching authority certificates (0/2)"
+        );
+
         // Check that we get the right list of missing docs.
         let missing = state.missing_docs();
         assert_eq!(missing.len(), 2); // We are missing two certificates.
@@ -1131,6 +1212,10 @@ mod test {
         let missing = state.missing_docs();
         assert_eq!(missing.len(), 1); // Now we're only missing one!
         assert!(missing.contains(&DocId::AuthCert(authcert_id_5a23())));
+        assert_eq!(
+            state.bootstrap_status().to_string(),
+            "fetching authority certificates (1/2)"
+        );
 
         // Now try to add the other from a download ... but fail
         // because we didn't ask for it.
@@ -1226,6 +1311,10 @@ mod test {
         }
         let retry = state.dl_config().unwrap();
         assert_eq!(&retry, DownloadScheduleConfig::default().retry_microdescs());
+        assert_eq!(
+            state.bootstrap_status().to_string(),
+            "fetching microdescriptors (0/4)"
+        );
 
         // Now check whether we're missing all the right microdescs.
         let missing = state.missing_docs();
@@ -1257,9 +1346,16 @@ mod test {
         let missing = state.missing_docs();
         assert_eq!(missing.len(), 3);
         assert!(!missing.contains(&DocId::Microdesc(md1)));
+        assert_eq!(
+            state.bootstrap_status().to_string(),
+            "fetching microdescriptors (1/4)"
+        );
 
         // Try adding the rest as if from a download.
         let mut req = tor_dirclient::request::MicrodescRequest::new();
+        // Clear this flag so that the test consensus won't expire the moment
+        // we're done.
+        state.expire_when_complete = false;
         let mut response = "".to_owned();
         for md_digest in [md2, md3, md4] {
             response.push_str(md_text.get(&md_digest).unwrap());
